@@ -28,6 +28,10 @@ try:
     import pandas  # type: ignore
 except ImportError:  # pragma: NO COVER
     pandas = None
+else:
+    import db_dtypes  # type: ignore # noqa
+
+import pyarrow  # type: ignore
 
 try:
     import geopandas  # type: ignore
@@ -43,18 +47,12 @@ except ImportError:
 else:
     _read_wkt = shapely.geos.WKTReader(shapely.geos.lgeos).read
 
-try:
-    import pyarrow  # type: ignore
-except ImportError:  # pragma: NO COVER
-    pyarrow = None
-
 import google.api_core.exceptions
 from google.api_core.page_iterator import HTTPIterator
 
 import google.cloud._helpers  # type: ignore
 from google.cloud.bigquery import _helpers
 from google.cloud.bigquery import _pandas_helpers
-from google.cloud.bigquery.exceptions import LegacyBigQueryStorageError
 from google.cloud.bigquery.schema import _build_schema_resource
 from google.cloud.bigquery.schema import _parse_schema_resource
 from google.cloud.bigquery.schema import _to_schema_fields
@@ -67,7 +65,6 @@ if typing.TYPE_CHECKING:  # pragma: NO COVER
     # they are not None, avoiding false "no attribute" errors.
     import pandas
     import geopandas
-    import pyarrow
     from google.cloud import bigquery_storage
     from google.cloud.bigquery.dataset import DatasetReference
 
@@ -84,10 +81,6 @@ _NO_SHAPELY_ERROR = (
     "The shapely library is not installed, please install "
     "shapely to use the geography_as_object option."
 )
-_NO_PYARROW_ERROR = (
-    "The pyarrow library is not installed, please install "
-    "pyarrow to use the to_arrow() function."
-)
 
 _TABLE_HAS_NO_SCHEMA = 'Table has no schema:  call "client.get_table()"'
 
@@ -213,7 +206,9 @@ class TableReference(_TableBase):
             dataset_ref.dataset_id,
         )
         _helpers._set_sub_prop(
-            self._properties, self._PROPERTY_TO_API_FIELD["table_id"], table_id,
+            self._properties,
+            self._PROPERTY_TO_API_FIELD["table_id"],
+            table_id,
         )
 
     @classmethod
@@ -274,6 +269,7 @@ class TableReference(_TableBase):
         project = resource["projectId"]
         dataset_id = resource["datasetId"]
         table_id = resource["tableId"]
+
         return cls(DatasetReference(project, dataset_id), table_id)
 
     def to_api_repr(self) -> dict:
@@ -375,7 +371,7 @@ class Table(_TableBase):
         "require_partition_filter": "requirePartitionFilter",
     }
 
-    def __init__(self, table_ref, schema=None):
+    def __init__(self, table_ref, schema=None) -> None:
         table_ref = _table_arg_to_table_ref(table_ref)
         self._properties = {"tableReference": table_ref.to_api_repr(), "labels": {}}
         # Let the @property do validation.
@@ -886,7 +882,9 @@ class Table(_TableBase):
 
         api_field = self._PROPERTY_TO_API_FIELD["mview_refresh_interval"]
         _helpers._set_sub_prop(
-            self._properties, [api_field, "refreshIntervalMs"], refresh_interval_ms,
+            self._properties,
+            [api_field, "refreshIntervalMs"],
+            refresh_interval_ms,
         )
 
     @property
@@ -1324,7 +1322,7 @@ class Row(object):
     # Choose unusual field names to try to avoid conflict with schema fields.
     __slots__ = ("_xxx_values", "_xxx_field_to_index")
 
-    def __init__(self, values, field_to_index):
+    def __init__(self, values, field_to_index) -> None:
         self._xxx_values = values
         self._xxx_field_to_index = field_to_index
 
@@ -1552,17 +1550,6 @@ class RowIterator(HTTPIterator):
         if self.max_results is not None:
             return False
 
-        try:
-            from google.cloud import bigquery_storage  # noqa: F401
-        except ImportError:
-            return False
-
-        try:
-            _helpers.BQ_STORAGE_VERSIONS.verify_version()
-        except LegacyBigQueryStorageError as exc:
-            warnings.warn(str(exc))
-            return False
-
         return True
 
     def _get_next_page_response(self):
@@ -1598,7 +1585,8 @@ class RowIterator(HTTPIterator):
         return self._total_rows
 
     def _maybe_warn_max_results(
-        self, bqstorage_client: Optional["bigquery_storage.BigQueryReadClient"],
+        self,
+        bqstorage_client: Optional["bigquery_storage.BigQueryReadClient"],
     ):
         """Issue a warning if BQ Storage client is not ``None`` with ``max_results`` set.
 
@@ -1661,15 +1649,8 @@ class RowIterator(HTTPIterator):
             pyarrow.RecordBatch:
                 A generator of :class:`~pyarrow.RecordBatch`.
 
-        Raises:
-            ValueError:
-                If the :mod:`pyarrow` library cannot be imported.
-
         .. versionadded:: 2.31.0
         """
-        if pyarrow is None:
-            raise ValueError(_NO_PYARROW_ERROR)
-
         self._maybe_warn_max_results(bqstorage_client)
 
         bqstorage_download = functools.partial(
@@ -1695,7 +1676,7 @@ class RowIterator(HTTPIterator):
     def to_arrow(
         self,
         progress_bar_type: str = None,
-        bqstorage_client: "bigquery_storage.BigQueryReadClient" = None,
+        bqstorage_client: Optional["bigquery_storage.BigQueryReadClient"] = None,
         create_bqstorage_client: bool = True,
     ) -> "pyarrow.Table":
         """[Beta] Create a class:`pyarrow.Table` by loading all pages of a
@@ -1724,8 +1705,7 @@ class RowIterator(HTTPIterator):
                 A BigQuery Storage API client. If supplied, use the faster BigQuery
                 Storage API to fetch rows from BigQuery. This API is a billable API.
 
-                This method requires the ``pyarrow`` and
-                ``google-cloud-bigquery-storage`` libraries.
+                This method requires ``google-cloud-bigquery-storage`` library.
 
                 This method only  exposes a subset of the capabilities of the
                 BigQuery Storage API.  For full access to all features
@@ -1746,14 +1726,8 @@ class RowIterator(HTTPIterator):
                 headers from the query results. The column headers are derived
                 from the destination table's schema.
 
-        Raises:
-            ValueError: If the :mod:`pyarrow` library cannot be imported.
-
         .. versionadded:: 1.17.0
         """
-        if pyarrow is None:
-            raise ValueError(_NO_PYARROW_ERROR)
-
         self._maybe_warn_max_results(bqstorage_client)
 
         if not self._validate_bqstorage(bqstorage_client, create_bqstorage_client):
@@ -1803,7 +1777,7 @@ class RowIterator(HTTPIterator):
 
     def to_dataframe_iterable(
         self,
-        bqstorage_client: "bigquery_storage.BigQueryReadClient" = None,
+        bqstorage_client: Optional["bigquery_storage.BigQueryReadClient"] = None,
         dtypes: Dict[str, Any] = None,
         max_queue_size: int = _pandas_helpers._MAX_QUEUE_SIZE_DEFAULT,  # type: ignore
     ) -> "pandas.DataFrame":
@@ -1814,8 +1788,7 @@ class RowIterator(HTTPIterator):
                 A BigQuery Storage API client. If supplied, use the faster
                 BigQuery Storage API to fetch rows from BigQuery.
 
-                This method requires the ``pyarrow`` and
-                ``google-cloud-bigquery-storage`` libraries.
+                This method requires ``google-cloud-bigquery-storage`` library.
 
                 This method only exposes a subset of the capabilities of the
                 BigQuery Storage API. For full access to all features
@@ -1880,11 +1853,10 @@ class RowIterator(HTTPIterator):
     # changes to job.QueryJob.to_dataframe()
     def to_dataframe(
         self,
-        bqstorage_client: "bigquery_storage.BigQueryReadClient" = None,
+        bqstorage_client: Optional["bigquery_storage.BigQueryReadClient"] = None,
         dtypes: Dict[str, Any] = None,
         progress_bar_type: str = None,
         create_bqstorage_client: bool = True,
-        date_as_object: bool = True,
         geography_as_object: bool = False,
     ) -> "pandas.DataFrame":
         """Create a pandas DataFrame by loading all pages of a query.
@@ -1894,8 +1866,7 @@ class RowIterator(HTTPIterator):
                 A BigQuery Storage API client. If supplied, use the faster
                 BigQuery Storage API to fetch rows from BigQuery.
 
-                This method requires the ``pyarrow`` and
-                ``google-cloud-bigquery-storage`` libraries.
+                This method requires ``google-cloud-bigquery-storage`` library.
 
                 This method only exposes a subset of the capabilities of the
                 BigQuery Storage API. For full access to all features
@@ -1935,12 +1906,6 @@ class RowIterator(HTTPIterator):
 
                 .. versionadded:: 1.24.0
 
-            date_as_object (Optional[bool]):
-                If ``True`` (default), cast dates to objects. If ``False``, convert
-                to datetime64[ns] dtype.
-
-                .. versionadded:: 1.26.0
-
             geography_as_object (Optional[bool]):
                 If ``True``, convert GEOGRAPHY data to :mod:`shapely`
                 geometry objects. If ``False`` (default), don't cast
@@ -1983,30 +1948,43 @@ class RowIterator(HTTPIterator):
             create_bqstorage_client=create_bqstorage_client,
         )
 
-        # When converting timestamp values to nanosecond precision, the result
+        # When converting date or timestamp values to nanosecond precision, the result
         # can be out of pyarrow bounds. To avoid the error when converting to
-        # Pandas, we set the timestamp_as_object parameter to True, if necessary.
-        types_to_check = {
-            pyarrow.timestamp("us"),
-            pyarrow.timestamp("us", tz=datetime.timezone.utc),
-        }
+        # Pandas, we set the date_as_object or timestamp_as_object parameter to True,
+        # if necessary.
+        date_as_object = not all(
+            self.__can_cast_timestamp_ns(col)
+            for col in record_batch
+            # Type can be date32 or date64 (plus units).
+            # See: https://arrow.apache.org/docs/python/api/datatypes.html
+            if str(col.type).startswith("date")
+        )
 
-        for column in record_batch:
-            if column.type in types_to_check:
-                try:
-                    column.cast("timestamp[ns]")
-                except pyarrow.lib.ArrowInvalid:
-                    timestamp_as_object = True
-                    break
+        timestamp_as_object = not all(
+            self.__can_cast_timestamp_ns(col)
+            for col in record_batch
+            # Type can be timestamp (plus units and time zone).
+            # See: https://arrow.apache.org/docs/python/api/datatypes.html
+            if str(col.type).startswith("timestamp")
+        )
+
+        if len(record_batch) > 0:
+            df = record_batch.to_pandas(
+                date_as_object=date_as_object,
+                timestamp_as_object=timestamp_as_object,
+                integer_object_nulls=True,
+                types_mapper=_pandas_helpers.default_types_mapper(
+                    date_as_object=date_as_object
+                ),
+            )
         else:
-            timestamp_as_object = False
-
-        extra_kwargs = {"timestamp_as_object": timestamp_as_object}
-
-        df = record_batch.to_pandas(date_as_object=date_as_object, **extra_kwargs)
+            # Avoid "ValueError: need at least one array to concatenate" on
+            # older versions of pandas when converting empty RecordBatch to
+            # DataFrame. See: https://github.com/pandas-dev/pandas/issues/41241
+            df = pandas.DataFrame([], columns=record_batch.schema.names)
 
         for column in dtypes:
-            df[column] = pandas.Series(df[column], dtype=dtypes[column])
+            df[column] = pandas.Series(df[column], dtype=dtypes[column], copy=False)
 
         if geography_as_object:
             for field in self.schema:
@@ -2015,6 +1993,15 @@ class RowIterator(HTTPIterator):
 
         return df
 
+    @staticmethod
+    def __can_cast_timestamp_ns(column):
+        try:
+            column.cast("timestamp[ns]")
+        except pyarrow.lib.ArrowInvalid:
+            return False
+        else:
+            return True
+
     # If changing the signature of this method, make sure to apply the same
     # changes to job.QueryJob.to_geodataframe()
     def to_geodataframe(
@@ -2023,7 +2010,6 @@ class RowIterator(HTTPIterator):
         dtypes: Dict[str, Any] = None,
         progress_bar_type: str = None,
         create_bqstorage_client: bool = True,
-        date_as_object: bool = True,
         geography_column: Optional[str] = None,
     ) -> "geopandas.GeoDataFrame":
         """Create a GeoPandas GeoDataFrame by loading all pages of a query.
@@ -2071,10 +2057,6 @@ class RowIterator(HTTPIterator):
 
                 This argument does nothing if ``bqstorage_client`` is supplied.
 
-            date_as_object (Optional[bool]):
-                If ``True`` (default), cast dates to objects. If ``False``, convert
-                to datetime64[ns] dtype.
-
             geography_column (Optional[str]):
                 If there are more than one GEOGRAPHY column,
                 identifies which one to use to construct a geopandas
@@ -2130,7 +2112,6 @@ class RowIterator(HTTPIterator):
             dtypes,
             progress_bar_type,
             create_bqstorage_client,
-            date_as_object,
             geography_as_object=True,
         )
 
@@ -2179,8 +2160,6 @@ class _EmptyRowIterator(RowIterator):
         Returns:
             pyarrow.Table: An empty :class:`pyarrow.Table`.
         """
-        if pyarrow is None:
-            raise ValueError(_NO_PYARROW_ERROR)
         return pyarrow.Table.from_arrays(())
 
     def to_dataframe(
@@ -2189,7 +2168,6 @@ class _EmptyRowIterator(RowIterator):
         dtypes=None,
         progress_bar_type=None,
         create_bqstorage_client=True,
-        date_as_object=True,
         geography_as_object=False,
     ) -> "pandas.DataFrame":
         """Create an empty dataframe.
@@ -2199,7 +2177,6 @@ class _EmptyRowIterator(RowIterator):
             dtypes (Any): Ignored. Added for compatibility with RowIterator.
             progress_bar_type (Any): Ignored. Added for compatibility with RowIterator.
             create_bqstorage_client (bool): Ignored. Added for compatibility with RowIterator.
-            date_as_object (bool): Ignored. Added for compatibility with RowIterator.
 
         Returns:
             pandas.DataFrame: An empty :class:`~pandas.DataFrame`.
@@ -2214,7 +2191,6 @@ class _EmptyRowIterator(RowIterator):
         dtypes=None,
         progress_bar_type=None,
         create_bqstorage_client=True,
-        date_as_object=True,
         geography_column: Optional[str] = None,
     ) -> "pandas.DataFrame":
         """Create an empty dataframe.
@@ -2224,7 +2200,6 @@ class _EmptyRowIterator(RowIterator):
             dtypes (Any): Ignored. Added for compatibility with RowIterator.
             progress_bar_type (Any): Ignored. Added for compatibility with RowIterator.
             create_bqstorage_client (bool): Ignored. Added for compatibility with RowIterator.
-            date_as_object (bool): Ignored. Added for compatibility with RowIterator.
 
         Returns:
             pandas.DataFrame: An empty :class:`~pandas.DataFrame`.
@@ -2285,13 +2260,7 @@ class _EmptyRowIterator(RowIterator):
 
         Returns:
             An iterator yielding a single empty :class:`~pyarrow.RecordBatch`.
-
-        Raises:
-            ValueError:
-                If the :mod:`pyarrow` library cannot be imported.
         """
-        if pyarrow is None:
-            raise ValueError(_NO_PYARROW_ERROR)
         return iter((pyarrow.record_batch([]),))
 
     def __iter__(self):
@@ -2322,7 +2291,7 @@ class PartitionRange(object):
             Private. Used to construct object from API resource.
     """
 
-    def __init__(self, start=None, end=None, interval=None, _properties=None):
+    def __init__(self, start=None, end=None, interval=None, _properties=None) -> None:
         if _properties is None:
             _properties = {}
         self._properties = _properties
@@ -2397,10 +2366,10 @@ class RangePartitioning(object):
             Private. Used to construct object from API resource.
     """
 
-    def __init__(self, range_=None, field=None, _properties=None):
+    def __init__(self, range_=None, field=None, _properties=None) -> None:
         if _properties is None:
             _properties = {}
-        self._properties = _properties
+        self._properties: Dict[str, Any] = _properties
 
         if range_ is not None:
             self.range_ = range_
@@ -2506,8 +2475,8 @@ class TimePartitioning(object):
 
     def __init__(
         self, type_=None, field=None, expiration_ms=None, require_partition_filter=None
-    ):
-        self._properties = {}
+    ) -> None:
+        self._properties: Dict[str, Any] = {}
         if type_ is None:
             self.type_ = TimePartitioningType.DAY
         else:
